from __future__ import unicode_literals

import django
from django.db.models import signals

from guardian import models as guardian_app
from guardian.conf import settings as guardian_settings
from guardian.compat import get_user_model
from guardian.compat import import_string


def get_init_anonymous_user(User):
    """
    Returns User model instance that would be referenced by guardian when
    permissions are checked against users that haven't signed into the system.

    :param User: User model - result of ``django.contrib.auth.get_user_model``.
    """
    kwargs = {
        User.USERNAME_FIELD: guardian_settings.ANONYMOUS_DEFAULT_USERNAME_VALUE
    }
    return User(**kwargs)


def create_anonymous_user(sender, **kwargs):
    """
    Creates anonymous User instance with id and username from settings.
    """
    User = get_user_model()
    try:
        User.objects.get(pk=guardian_settings.ANONYMOUS_USER_ID)
    except User.DoesNotExist:
        if django.VERSION >= (1, 5):
            retrieve_anonymous_functon = import_string(
                guardian_settings.GET_INIT_ANONYMOUS_USER)
            user = retrieve_anonymous_functon(User)
            # Always set pk to the one pointed at settings
            user.pk = guardian_settings.ANONYMOUS_USER_ID
            user.save()
        else:
            User.objects.create(pk=guardian_settings.ANONYMOUS_USER_ID,
                username=guardian_settings.ANONYMOUS_DEFAULT_USERNAME_VALUE)

# Only create an anonymous user if support is enabled.
if guardian_settings.ANONYMOUS_USER_ID is not None:
    signals.post_syncdb.connect(create_anonymous_user, sender=guardian_app,
        dispatch_uid="guardian.management.create_anonymous_user")




from __future__ import absolute_import
from __future__ import unicode_literals

from guardian.compat import get_user_model
from guardian.compat import mock
from guardian.compat import unittest
from guardian.management import create_anonymous_user
import django


mocked_get_init_anon = mock.Mock()


class TestGetAnonymousUser(unittest.TestCase):

    @unittest.skipUnless(django.VERSION >= (1, 5), "Django >= 1.5 only")
    @mock.patch('guardian.management.guardian_settings')
    def test_uses_custom_function(self, guardian_settings):
        path = 'guardian.testapp.tests.management_test.mocked_get_init_anon'
        guardian_settings.GET_INIT_ANONYMOUS_USER = path
        guardian_settings.ANONYMOUS_USER_ID = 219
        User = get_user_model()

        anon = mocked_get_init_anon.return_value = mock.Mock()

        create_anonymous_user('sender')

        mocked_get_init_anon.assert_called_once_with(User)

        self.assertEqual(anon.pk, 219)
        anon.save.assert_called_once_with()



from __future__ import unicode_literals
# handler404 and handler500 are needed for admin tests
from guardian.compat import include, patterns, handler404, handler500 # pyflakes:ignore
from guardian.mixins import PermissionRequiredMixin
from django.contrib import admin
from django import get_version as django_get_version

if (django_get_version() >= "1.5"):
    from django.views.generic import View

admin.autodiscover()

urlpatterns = patterns('',
    (r'^admin/', include(admin.site.urls)),
)

if (django_get_version() >= "1.5"):

    class testClassRedirectView(PermissionRequiredMixin, View):
        permission_required = 'testapp.change_project'

    urlpatterns += patterns('',
        (r'^accounts/login/', 'django.contrib.auth.views.login', {'template_name': 'blank.html'}),
        (r'^permission_required/', testClassRedirectView.as_view()),
    )
